@@ -153,7 +153,7 @@ CFLACTag::CFLACTag(const TCHAR * pFilename)
                             // read data
                             if ((32 + nMIMELength + nDescriptionLength + m_nPictureBytes) <= nBufferBytes)
                             {
-                                m_spPicture.Assign(new char [m_nPictureBytes], true);
+                                m_spPicture.AllocateArray(m_nPictureBytes);
                                 memcpy(m_spPicture, &spPicture[32 + nMIMELength + nDescriptionLength], m_nPictureBytes);
                             }
                         }
