@@ -25,8 +25,8 @@ CAPEDecompressOld::CAPEDecompressOld(int * pErrorCode, CAPEInfo * pAPEInfo, int
     m_nCurrentBlock = 0;
 
     // set the "real" start and finish blocks
-    m_nStartBlock = (nStartBlock < 0) ? 0 : ape_min(nStartBlock, static_cast<int>(m_spAPEInfo->GetInfo(APE_INFO_TOTAL_BLOCKS)));
-    m_nFinishBlock = (nFinishBlock < 0) ? static_cast<int>(m_spAPEInfo->GetInfo(APE_INFO_TOTAL_BLOCKS)) : ape_min(nFinishBlock, static_cast<int>(m_spAPEInfo->GetInfo(APE_INFO_TOTAL_BLOCKS)));
+    m_nStartBlock = (nStartBlock < 0) ? 0 : APE_MIN(nStartBlock, static_cast<int>(m_spAPEInfo->GetInfo(APE_INFO_TOTAL_BLOCKS)));
+    m_nFinishBlock = (nFinishBlock < 0) ? static_cast<int>(m_spAPEInfo->GetInfo(APE_INFO_TOTAL_BLOCKS)) : APE_MIN(nFinishBlock, static_cast<int>(m_spAPEInfo->GetInfo(APE_INFO_TOTAL_BLOCKS)));
     m_bIsRanged = (m_nStartBlock != 0) || (m_nFinishBlock != static_cast<int>(m_spAPEInfo->GetInfo(APE_INFO_TOTAL_BLOCKS)));
 
     // version check (this implementation only works with 3.92 and earlier files)
@@ -49,6 +49,11 @@ CAPEDecompressOld::~CAPEDecompressOld()
 {
 }
 
+int CAPEDecompressOld::SetNumberOfThreads(int)
+{
+    return 1;
+}
+
 int CAPEDecompressOld::InitializeDecompressor()
 {
     // check if we have anything to do
@@ -59,8 +64,8 @@ int CAPEDecompressOld::InitializeDecompressor()
     RETURN_ON_ERROR(m_UnMAC.Initialize(this))
 
     const int64 nMaximumDecompressedFrameBytes = m_nBlockAlign * static_cast<intn>(GetInfo(APE_INFO_BLOCKS_PER_FRAME));
-    const int64 nTotalBufferBytes = ape_max(65536, (nMaximumDecompressedFrameBytes + 16) * 2);
-    m_spBuffer.Assign(new unsigned char [static_cast<unsigned int>(nTotalBufferBytes)], true);
+    const int64 nTotalBufferBytes = APE_MAX(65536, (nMaximumDecompressedFrameBytes + 16) * 2);
+    m_spBuffer.AllocateArray(nTotalBufferBytes);
     if (m_spBuffer == APE_NULL)
         return ERROR_INSUFFICIENT_MEMORY;
 
@@ -79,7 +84,7 @@ int CAPEDecompressOld::GetData(unsigned char * pBuffer, int64 nBlocks, int64 * p
 
     // cap
     const int64 nBlocksUntilFinish = m_nFinishBlock - m_nCurrentBlock;
-    nBlocks = ape_min(nBlocks, nBlocksUntilFinish);
+    nBlocks = APE_MIN(nBlocks, nBlocksUntilFinish);
 
     int64 nBlocksRetrieved = 0;
 
@@ -92,7 +97,7 @@ int CAPEDecompressOld::GetData(unsigned char * pBuffer, int64 nBlocks, int64 * p
     {
         // empty the buffer
         const int64 nBytesAvailable = m_nBufferTail;
-        const int64 nIntialBytes = ape_min(nBytesLeft, nBytesAvailable);
+        const int64 nIntialBytes = APE_MIN(nBytesLeft, nBytesAvailable);
         if (nIntialBytes > 0)
         {
             memcpy(&pBuffer[nTotalBytesNeeded - nBytesLeft], &m_spBuffer[0], static_cast<size_t>(nIntialBytes));
@@ -108,11 +113,10 @@ int CAPEDecompressOld::GetData(unsigned char * pBuffer, int64 nBlocks, int64 * p
         // decode more
         if (nBytesLeft > 0)
         {
-            nBlocksDecoded = m_UnMAC.DecompressFrame(&m_spBuffer[m_nBufferTail], static_cast<int32>(m_nCurrentFrame++));
-            if (nBlocksDecoded == -1)
-            {
-                return ERROR_UNDEFINED;
-            }
+            int nErrorCode = ERROR_UNDEFINED;
+            nBlocksDecoded = m_UnMAC.DecompressFrame(&m_spBuffer[m_nBufferTail], static_cast<int32>(m_nCurrentFrame++), &nErrorCode);
+            if (nBlocksDecoded < 0)
+                return nErrorCode;
             m_nBufferTail += (nBlocksDecoded * m_nBlockAlign);
         }
     }
@@ -150,18 +154,16 @@ int CAPEDecompressOld::Seek(int64 nBlockOffset)
 
     // skip necessary blocks
     const int64 nMaximumDecompressedFrameBytes = m_nBlockAlign * static_cast<int>(GetInfo(APE_INFO_BLOCKS_PER_FRAME));
-    CSmartPtr<unsigned char> spTempBuffer;
-    spTempBuffer.Assign(new unsigned char[static_cast<size_t>(nMaximumDecompressedFrameBytes + 16)], true);
+    CSmartPtr<unsigned char> spTempBuffer(nMaximumDecompressedFrameBytes + 16);
     ZeroMemory(spTempBuffer.GetPtr(), static_cast<size_t>(nMaximumDecompressedFrameBytes + 16));
 
     m_nCurrentFrame = nBaseFrame;
 
-    const intn nBlocksDecoded = m_UnMAC.DecompressFrame(spTempBuffer.GetPtr(), static_cast<int32>(m_nCurrentFrame++));
+    int nErrorCode = ERROR_UNDEFINED;
+    const intn nBlocksDecoded = m_UnMAC.DecompressFrame(spTempBuffer.GetPtr(), static_cast<int32>(m_nCurrentFrame++), &nErrorCode);
 
-    if (nBlocksDecoded == -1)
-    {
-        return ERROR_UNDEFINED;
-    }
+    if (nBlocksDecoded < 0)
+        return nErrorCode;
 
     const int64 nBytesToKeep = (nBlocksDecoded * m_nBlockAlign) - nBytesToSkip;
     memcpy(&m_spBuffer[m_nBufferTail], &spTempBuffer[nBytesToSkip], static_cast<size_t>(nBytesToKeep));
@@ -254,9 +256,11 @@ int64 CAPEDecompressOld::GetInfo(APE_DECOMPRESS_FIELDS Field, int64 nParam1, int
             }
             else
             {
-                WAVEFORMATEX wfeFormat; APE_CLEAR(wfeFormat);
+                WAVEFORMATEX wfeFormat;
                 GetInfo(APE_INFO_WAVEFORMATEX, POINTER_TO_INT64(&wfeFormat), 0);
-                WAVE_HEADER WAVHeader; FillWaveHeader(&WAVHeader,
+
+                WAVE_HEADER WAVHeader;
+                FillWaveHeader(&WAVHeader,
                     (m_nFinishBlock - m_nStartBlock) * static_cast<intn>(GetInfo(APE_INFO_BLOCK_ALIGN)),
                     &wfeFormat, 0);
                 memcpy(pBuffer, &WAVHeader, sizeof(WAVE_HEADER));
